@@ -253,6 +253,62 @@ class KronInputWrapper(nn.Module):
 
         return out
 
+# convolutional blocks
+
+def same_padding(kernel, dilation):
+    return (kernel + (kernel - 1) * (dilation - 1)) // 4
+
+class ReshapeForConv(nn.Module):
+    def __init__(self, fn):
+        super().__init__()
+        self.fn = fn
+
+    def forward(self, x, shape, mask = None, **kwargs):
+        x = x.view(*shape, -1)
+        x = rearrange(x, 'b h w c -> b c h w')
+
+        if exists(mask):
+            mask = mask.view(*shape)
+
+        x = self.fn(x, mask = mask)
+        x = rearrange(x, 'b c h w -> b (h w) c')
+        return x
+
+class HybridDimensionalConvBlock(nn.Module):
+    def __init__(
+        self,
+        *,
+        dim,
+        kernels,
+        dilations,
+        expansion_factor = 2,
+        glu_gate = False
+    ):
+        super().__init__()
+        self.convs = nn.ModuleList([])
+        dilations = cast_tuple(dilations, len(kernels))
+
+        for kernel, dilation in zip(kernels, dilations):
+            padding = tuple(map(partial(same_padding, dilation = dilation), kernel))
+            mult = 2 if glu_gate else 1
+
+            self.convs.append(nn.Sequential(
+                nn.Conv2d(dim, dim * expansion_factor * mult, kernel, padding = padding),
+                nn.GELU() if not glu_gate else nn.GLU(dim = 1),
+                nn.Conv2d(dim * expansion_factor, dim, kernel, padding = padding),
+            ))
+
+    def forward(self, x, mask = None):
+        if exists(mask):
+            mask = rearrange(mask, 'b h w -> b () h w')
+            x.masked_fill_(~mask, 0.)
+
+        out = []
+        for conv in self.convs:
+            out.append(conv(x))
+
+        return sum(out)
+
 # feed forward
 
 class DepthWiseConv2d(nn.Module):
@@ -951,7 +1007,7 @@ class Alphafold2(nn.Module):
                 raise ValueError(f'cannot find attention type {attn_type}')
 
             layers.append(nn.ModuleList([
-                prenorm(InterceptAxialAttention(tensor_slice, AxialAttention(dim = dim, template_axial_attn = template_axial_attn, seq_len = max_seq_len, heads = heads, dim_head = dim_head, dropout = attn_dropout, sparse_attn = sparse_self_attn, row_attn = row_attn, col_attn = col_attn))),
+                prenorm(InterceptAxialAttention(tensor_slice, AxialAttention(dim = dim, template_axial_attn = template_axial_attn, seq_len = max_seq_len, heads = heads, dim_head = dim_head, dropout = attn_dropout, sparse_attn = layer_sparse_attn, row_attn = row_attn, col_attn = col_attn))),
                 prenorm(InterceptFeedForward(ff_tensor_slice, ff = LocalFeedForward(dim = dim, hidden_dim = dim * 4, dropout = ff_dropout))),
                 prenorm(AxialAttention(dim = dim, seq_len = max_seq_len, heads = heads, dim_head = dim_head, dropout = attn_dropout, tie_row_attn = msa_tie_row_attn, row_attn = row_attn, col_attn = col_attn)),
                 prenorm(FeedForward(dim = dim, dropout = ff_dropout)),
