@@ -6,6 +6,8 @@ MAX_NUM_MSA = 20
 MAX_NUM_TEMPLATES = 10
 NUM_AMINO_ACIDS = 21
 NUM_EMBEDDS_TR = 1280 # best esm model 
+NUM_EMBEDDS_T5 = 1024 # best t5 model 
+NUM_COORDS_PER_RES = 14
 
 DISTOGRAM_BUCKETS = 37
 THETA_BUCKETS = 25
